/*  Copyright (c) 1998-2000 IBK-Landquart-Switzerland. All rights reserved.
 *
 *  Module      :  Embedd.cpp
 *  Application :  Sample to embedd CSS as macro language with CPP-API
 *  Author      :  Peter Koch, IBK
 *
 *  Date        Description                                 Who
 *  --------------------------------------------------------------------------
 *  Feb 2000    First release                               P.Koch, IBK
 */
#include <stdio.h>
#include <strstrea.h>
#include <KCss.hpp>

static KCss* css(0);                     // css object ptr
static Boolean cssOk(false);             // ok flag
static IString module("Embedd.exe");     // module name

/*
 * c h e c k N u m b e r
 *
 * Check if string represents a number
 */
static Boolean checkNumber(const IString& str)
{
   char *s = str;
   if (*s=='-' || *s=='+') s++;
   Boolean any(false);
   while ('0'<=*s && *s<='9') { s++; any = true; }
   if (*s=='.') s++;
   while ('0'<=*s && *s<='9') s++;
   return any && *s == 0;
} // checkNumber

/*
 * a v e r a g e
 *
 * Sample CSS function calculating the average of numbers
 */
static IString average(KCss* aCss)
{
   // get actual # of arguments
   int argCount = aCss->get("argCount").asInt();

   // calculate sum of all arguments
   double sum(0.0);
   for (int i = 0; i < argCount; i++) {
      // get argument
      IString val = aCss->get("p"+IString(i+1));

      // check for number
      if (!checkNumber(val))
         throw IException("argument "+IString(i+1)+" is no number!");

      sum += val.asDouble();
   } // for

   // return result
   return IString(sum / argCount);
} // average

int main()
{
   int ret(0);
   try {
      cout << "creating css object" << endl;
      css = new KCss();
      cssOk = true;

      cout << endl << "get css version" << endl;
      cout << "  css version is " << css->get("cssVersion") << endl;

      cout << endl << "load c++ function 'average'" << endl;
      css->addFunc(
         module,
         "average(const p1, [const p2, const p3, const p4, const p5])",
         average);

      cout << endl << "call 'average' from c++" << endl;
      IString res = css->call(module, "average", 3, "3", "12", "17");
      cout << "  result = " << res << endl;

      cout << endl << "compile a script from memory" << endl;
      istrstream str(
         "#loadLibrary 'KcSysLib'\n"
         ""
         "check()\n"
         "{\n"
         "   sysLog('the average of 3,5,12,7 is '\n"
         "          |average(3,5,12,7)\n"
         "   );\n"
         "}\n"
      );
      css->loadScript(module, &str);

      cout << endl << "call 'check' within compiled script" << endl;
      css->call(module, "check", 0, 0);

      cout << endl << "deleting css object" << endl;
      delete css;
   } // try
   catch (IException err) {
      for (int i = err.textCount()-1; i >= 0; i--)
         if (*err.text(i)) cerr << err.text(i) << endl;
      if (cssOk) delete css;
      ret = 1;
   } // catch
   return ret;
} // main

